#include <iostream>
#include <sstream>
#include <iomanip>
#include <utility>
#include <cmath>
#include <cassert>
#include "matrix.h"

void Matrix::swapRows(int i0, int i1) {
    assert(0 <= i0 && i0 < m);
    assert(0 <= i1 && i1 < m);
    if (i0 == i1)
        return;
    for (int j = 0; j < n; ++j) {
        double tmp = at(i0, j);
        at(i0, j) = at(i1, j);
        at(i1, j) = (-tmp);
    }
}

void Matrix::addRows(int i, int k, double coeff) {
    assert(0 <= i && i < m);
    assert(0 <= k && k < m);
    assert(i != k);
    for (int j = 0; j < n; ++j) {
        at(i, j) += at(k, j)*coeff;
    }
}

int Matrix::gauss() {
    int i = 0;
    int j = 0;
    while (i < m && j < n) {
        // Ищем максимальный по модулю элемент
        // в столбце j, начиная со строки i
        double maxElem = fabs(at(i, j));
        double maxLine = i;
        for (int k = i+1; k < m; ++k) {
            if (fabs(at(k, j)) > maxElem) {
                maxElem = fabs(at(k, j));
                maxLine = k;
            }
        }
        if (maxElem <= MATRIX_EPS) {
            ++j;
            continue;
        }
        
        // Меняем местами строки с индексами i, maxLine
        if (i != maxLine)
            swapRows(i, maxLine);
        for (int k = i + 1; k < m; ++k) {
            addRows(k, i, -at(k, j)/at(i, j));
        }
        ++i; ++j;
    }
    return i;
}

std::ostream& operator<<(std::ostream& s, const Matrix& a) {
    std::stringstream txt;
    txt << std::fixed << std::setprecision(4);
    int maxlen = 0;
    for (int i = 0; i < a.rows(); ++i) {
        for (int j = 0; j < a.rows(); ++j) {
            txt.clear();
            txt.str("");
            txt << a.at(i, j);
            int l = int(txt.str().size());
            if (l > maxlen)
                maxlen = l;
        }
    }
    
    for (int i = 0; i < a.rows(); ++i) {
        for (int j = 0; j < a.rows(); ++j) {
            if (j > 0)
                s << " ";
            txt.clear();
            txt.str("");
            txt << a.at(i, j);
            int l = int(txt.str().size());
            for (int k = 0; k < maxlen - l; ++k)
                s << " ";
            s << txt.str();
        }
        s << std::endl;
    }
    
    return s;
}

std::istream& operator>>(std::istream& s, Matrix& a) {
    for (int i = 0; i < a.rows(); ++i) {
        for (int j = 0; j < a.rows(); ++j) {
            s >> a.at(i, j);
            if (!s.good())
                return s;
        }
    }
    return s;
}