#ifndef MATRIX_H
#define MATRIX_H 1

#include <vector>
#include <iostream>
#include <stdexcept>

constexpr double MATRIX_EPS = 1e-8;

class Matrix {
private:
    int m;  // Number of rows
    int n;  // Number of columns
    std::vector<double> elems;
public:
    Matrix(int r = 1, int c = 1):
        m(r),
        n(c),
        elems(m*n)
    {}
    
    int rows() const { return m; }
    int columns() const { return n; }
    int cols() const { return columns(); }
    
    // double x = a.at(i, j);
    double at(int i, int j) const {
        if (i < 0 || i >= m || j < 0 || j >= n)
            throw std::range_error("matrix index out of bounds");
        return elems[i*n + j];
    }
    
    // a.at(i, j) = 0.75;
    double& at(int i, int j) {
        if (i < 0 || i >= m || j < 0 || j >= n)
            throw std::range_error("matrix index out of bounds");
        return elems[i*n + j];
    }
    
    // x = a[i][j];
    const double* operator[](int i) const {
        return elems.data() + i*n;
    } 
    
    // a[i][j] = 0.5;
    double* operator[](int i) {
        return elems.data() + i*n;
    }

    void resize(int r, int c) {
        m = r; n = c;
        elems.resize(m*n);
    }
    
    int gauss();    // Returns rank of matrix
    double det() const;
    Matrix inverse() const;
    
    void swapRows(int i0, int i1);
    void addRows(int i, int k, double coeff);
};
   
std::ostream& operator<<(std::ostream& s, const Matrix& a);

std::istream& operator>>(std::istream& s, Matrix& a);

#endif   
